package de.meinefirma.meinprojekt;

import java.io.*;
import java.net.SocketException;
import java.util.*;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.ftpserver.*;
import org.apache.ftpserver.ftplet.*;
import org.apache.ftpserver.listener.ListenerFactory;
import org.apache.ftpserver.usermanager.*;
import org.apache.ftpserver.usermanager.impl.*;

/**
 * FTP-Utilities, basierend auf:<br>
 * Apache FTPClient:
 * {@link http://commons.apache.org/net/api/org/apache/commons/net/ftp/FTPClient.html}
 * Apache FtpServer:
 * {@link http://www.jarvana.com/jarvana/view/org/apache/ftpserver/ftpserver-core/1.0.4/ftpserver-core-1.0.4-javadoc.jar!/org/apache/ftpserver/FtpServer.html}
 */
public class FtpUtils
{
   /**
    * Erzeuge FTP-Server
    * @param ftpPort           FTP-Port, z.B. 2121
    * @param ftpHomeDir        FTP-Verzeichnis, z.B. "target/FtpHome"
    * @param readUserName      leseberechtigter Benutzer-Name
    * @param readUserPwd       leseberechtigter Benutzer-Passwort
    * @param writeUserName     schreibberechtigter Benutzer-Name
    * @param writeUserPwd      schreibberechtigter Benutzer-Passwort
    * @param ftpUsersPropsFile kann null sein, oder z.B. "target/FtpUsers.properties"
    */
   public static FtpServer createFtpServer( int ftpPort, String ftpHomeDir,
         String readUserName, String readUserPwd, String writeUserName, String writeUserPwd,
         String ftpUsersPropsFile ) throws FtpException, IOException
   {
      File fhd = new File( ftpHomeDir );
      if( !fhd.exists() ) fhd.mkdirs();

      ListenerFactory listenerFactory = new ListenerFactory();
      listenerFactory.setPort( ftpPort );

      PropertiesUserManagerFactory userManagerFactory = new PropertiesUserManagerFactory();
      userManagerFactory.setPasswordEncryptor( new SaltedPasswordEncryptor() );
      if( ftpUsersPropsFile != null && ftpUsersPropsFile.trim().length() > 0 ) {
         File upf = new File( ftpUsersPropsFile );
         if( !upf.exists() ) upf.createNewFile();
         userManagerFactory.setFile( upf );
      }

      // Einen Nur-Lese-User und einen User mit Schreibberechtigung anlegen:
      UserManager userManager = userManagerFactory.createUserManager();
      BaseUser userRd = new BaseUser();
      BaseUser userWr = new BaseUser();
      userRd.setName( readUserName );
      userRd.setPassword( readUserPwd );
      userRd.setHomeDirectory( ftpHomeDir );
      userWr.setName( writeUserName );
      userWr.setPassword( writeUserPwd );
      userWr.setHomeDirectory( ftpHomeDir );
      List<Authority> authorities = new ArrayList<Authority>();
      authorities.add( new WritePermission() );
      userWr.setAuthorities( authorities );
      userManager.save( userRd );
      userManager.save( userWr );

      FtpServerFactory serverFactory = new FtpServerFactory();
      serverFactory.addListener( "default", listenerFactory.createListener() );
      serverFactory.setUserManager( userManager );
      return serverFactory.createServer();
   }

   /**
    * FTP-Client-Upload
    * @return true falls ok
    */
   public static boolean upload( String localSourceFile, String remoteResultFile,
         String host, int port, String usr, String pwd, boolean showMessages ) throws SocketException, IOException
   {
      FTPClient       ftpClient = new FTPClient();
      FileInputStream fis = null;
      boolean         resultOk = true;

      try {
         ftpClient.connect( host, port );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         resultOk &= ftpClient.login( usr, pwd );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         fis = new FileInputStream( localSourceFile );
         resultOk &= ftpClient.storeFile( remoteResultFile, fis );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         resultOk &= ftpClient.logout();
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
      } finally {
         try { if( fis != null ) fis.close(); } catch( IOException e ) {}
         ftpClient.disconnect();
      }

      return resultOk;
   }

   /**
    * FTP-Client-Download
    * @return true falls ok
    */
   public static boolean download( String localResultFile, String remoteSourceFile,
         String host, int port, String usr, String pwd, boolean showMessages ) throws SocketException, IOException
   {
      FTPClient        ftpClient = new FTPClient();
      FileOutputStream fos = null;
      boolean          resultOk = true;

      try {
         ftpClient.connect( host, port );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         resultOk &= ftpClient.login( usr, pwd );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         fos = new FileOutputStream( localResultFile );
         resultOk &= ftpClient.retrieveFile( remoteSourceFile, fos );
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
         resultOk &= ftpClient.logout();
         if( showMessages ) System.out.println( ftpClient.getReplyString() );
      } finally {
         try { if( fos != null ) fos.close(); } catch( IOException e ) {}
         ftpClient.disconnect();
      }

      return resultOk;
   }

   /**
    * FTP-Dateienliste
    * @return String-Array der Dateinamen auf dem FTP-Server
    */
   public static String[] list( String host, int port, String usr, String pwd ) throws SocketException, IOException
   {
      FTPClient ftpClient = new FTPClient();
      String[]  filenameList;

      try {
         ftpClient.connect( host, port );
         ftpClient.login( usr, pwd );
         filenameList = ftpClient.listNames();
         ftpClient.logout();
      } finally {
         ftpClient.disconnect();
      }

      return filenameList;
   }
}
